@@ -1,9 +1,23 @@
+import json
+from binascii import b2a_hex
+try:
+    from django.utils.six.moves import cPickle as pickle
+except ImportError:
+    import pickle
 import unittest
-from django.contrib.gis.gdal import OGRGeometry, OGRGeomType, \
-    OGRException, OGRIndexError, SpatialReference, CoordTransform, \
-    GDAL_VERSION
-from django.contrib.gis.geometry.test_data import TestDataMixin
+from unittest import skipUnless
 
+from django.contrib.gis.gdal import HAS_GDAL
+from django.contrib.gis.geometry.test_data import TestDataMixin
+from django.utils.six.moves import xrange
+
+if HAS_GDAL:
+    from django.contrib.gis.gdal import (OGRGeometry, OGRGeomType,
+        OGRException, OGRIndexError, SpatialReference, CoordTransform,
+        GDAL_VERSION)
+
+
+@skipUnless(HAS_GDAL, "GDAL is required")
 class OGRGeomTest(unittest.TestCase, TestDataMixin):
     "This tests the OGR Geometry."
 
@@ -11,15 +25,12 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
         "Testing OGRGeomType object."
 
         # OGRGeomType should initialize on all these inputs.
-        try:
-            g = OGRGeomType(1)
-            g = OGRGeomType(7)
-            g = OGRGeomType('point')
-            g = OGRGeomType('GeometrycollectioN')
-            g = OGRGeomType('LINearrING')
-            g = OGRGeomType('Unknown')
-        except:
-            self.fail('Could not create an OGRGeomType object!')
+        OGRGeomType(1)
+        OGRGeomType(7)
+        OGRGeomType('point')
+        OGRGeomType('GeometrycollectioN')
+        OGRGeomType('LINearrING')
+        OGRGeomType('Unknown')
 
         # Should throw TypeError on this input
         self.assertRaises(OGRException, OGRGeomType, 23)
@@ -27,19 +38,20 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
         self.assertRaises(OGRException, OGRGeomType, 9)
 
         # Equivalence can take strings, ints, and other OGRGeomTypes
-        self.assertEqual(True, OGRGeomType(1) == OGRGeomType(1))
-        self.assertEqual(True, OGRGeomType(7) == 'GeometryCollection')
-        self.assertEqual(True, OGRGeomType('point') == 'POINT')
-        self.assertEqual(False, OGRGeomType('point') == 2)
-        self.assertEqual(True, OGRGeomType('unknown') == 0)
-        self.assertEqual(True, OGRGeomType(6) == 'MULtiPolyGON')
-        self.assertEqual(False, OGRGeomType(1) != OGRGeomType('point'))
-        self.assertEqual(True, OGRGeomType('POINT') != OGRGeomType(6))
+        self.assertEqual(OGRGeomType(1), OGRGeomType(1))
+        self.assertEqual(OGRGeomType(7), 'GeometryCollection')
+        self.assertEqual(OGRGeomType('point'), 'POINT')
+        self.assertNotEqual(OGRGeomType('point'), 2)
+        self.assertEqual(OGRGeomType('unknown'), 0)
+        self.assertEqual(OGRGeomType(6), 'MULtiPolyGON')
+        self.assertEqual(OGRGeomType(1), OGRGeomType('point'))
+        self.assertNotEqual(OGRGeomType('POINT'), OGRGeomType(6))
 
         # Testing the Django field name equivalent property.
         self.assertEqual('PointField', OGRGeomType('Point').django)
+        self.assertEqual('GeometryField', OGRGeomType('Geometry').django)
         self.assertEqual('GeometryField', OGRGeomType('Unknown').django)
-        self.assertEqual(None, OGRGeomType('none').django)
+        self.assertIsNone(OGRGeomType('none').django)
 
         # 'Geometry' initialization implies an unknown geometry type.
         gt = OGRGeomType('Geometry')
@@ -49,8 +61,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
     def test00b_geomtype_25d(self):
         "Testing OGRGeomType object with 25D types."
         wkb25bit = OGRGeomType.wkb25bit
-        self.failUnless(OGRGeomType(wkb25bit + 1) == 'Point25D')
-        self.failUnless(OGRGeomType('MultiLineString25D') == (5 + wkb25bit))
+        self.assertEqual(OGRGeomType(wkb25bit + 1), 'Point25D')
+        self.assertEqual(OGRGeomType('MultiLineString25D'), (5 + wkb25bit))
         self.assertEqual('GeometryCollectionField', OGRGeomType('GeometryCollection25D').django)
 
     def test01a_wkt(self):
@@ -85,39 +97,40 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
         "Testing HEX input/output."
         for g in self.geometries.hex_wkt:
             geom1 = OGRGeometry(g.wkt)
-            self.assertEqual(g.hex, geom1.hex)
+            self.assertEqual(g.hex.encode(), geom1.hex)
             # Constructing w/HEX
             geom2 = OGRGeometry(g.hex)
             self.assertEqual(geom1, geom2)
 
     def test01d_wkb(self):
         "Testing WKB input/output."
-        from binascii import b2a_hex
         for g in self.geometries.hex_wkt:
             geom1 = OGRGeometry(g.wkt)
             wkb = geom1.wkb
-            self.assertEqual(b2a_hex(wkb).upper(), g.hex)
+            self.assertEqual(b2a_hex(wkb).upper(), g.hex.encode())
             # Constructing w/WKB.
             geom2 = OGRGeometry(wkb)
             self.assertEqual(geom1, geom2)
 
     def test01e_json(self):
         "Testing GeoJSON input/output."
-        from django.contrib.gis.gdal.prototypes.geom import GEOJSON
-        if not GEOJSON: return
         for g in self.geometries.json_geoms:
             geom = OGRGeometry(g.wkt)
             if not hasattr(g, 'not_equal'):
-                self.assertEqual(g.json, geom.json)
-                self.assertEqual(g.json, geom.geojson)
+                # Loading jsons to prevent decimal differences
+                self.assertEqual(json.loads(g.json), json.loads(geom.json))
+                self.assertEqual(json.loads(g.json), json.loads(geom.geojson))
             self.assertEqual(OGRGeometry(g.wkt), OGRGeometry(geom.json))
+        # Test input with some garbage content (but valid json) (#15529)
+        geom = OGRGeometry('{"type": "Point", "coordinates": [ 100.0, 0.0 ], "other": "<test>"}')
+        self.assertIsInstance(geom, OGRGeometry)
 
     def test02_points(self):
         "Testing Point objects."
 
-        prev = OGRGeometry('POINT(0 0)')
+        OGRGeometry('POINT(0 0)')
         for p in self.geometries.points:
-            if not hasattr(p, 'z'): # No 3D
+            if not hasattr(p, 'z'):  # No 3D
                 pnt = OGRGeometry(p.wkt)
                 self.assertEqual(1, pnt.geom_type)
                 self.assertEqual('POINT', pnt.geom_name)
@@ -128,15 +141,15 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
     def test03_multipoints(self):
         "Testing MultiPoint objects."
         for mp in self.geometries.multipoints:
-            mgeom1 = OGRGeometry(mp.wkt) # First one from WKT
+            mgeom1 = OGRGeometry(mp.wkt)  # First one from WKT
             self.assertEqual(4, mgeom1.geom_type)
             self.assertEqual('MULTIPOINT', mgeom1.geom_name)
-            mgeom2 = OGRGeometry('MULTIPOINT') # Creating empty multipoint
+            mgeom2 = OGRGeometry('MULTIPOINT')  # Creating empty multipoint
             mgeom3 = OGRGeometry('MULTIPOINT')
             for g in mgeom1:
-                mgeom2.add(g) # adding each point from the multipoints
-                mgeom3.add(g.wkt) # should take WKT as well
-            self.assertEqual(mgeom1, mgeom2) # they should equal
+                mgeom2.add(g)  # adding each point from the multipoints
+                mgeom3.add(g.wkt)  # should take WKT as well
+            self.assertEqual(mgeom1, mgeom2)  # they should equal
             self.assertEqual(mgeom1, mgeom3)
             self.assertEqual(mp.coords, mgeom2.coords)
             self.assertEqual(mp.n_p, mgeom2.point_count)
@@ -150,8 +163,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             self.assertEqual('LINESTRING', linestr.geom_name)
             self.assertEqual(ls.n_p, linestr.point_count)
             self.assertEqual(ls.coords, linestr.tuple)
-            self.assertEqual(True, linestr == OGRGeometry(ls.wkt))
-            self.assertEqual(True, linestr != prev)
+            self.assertEqual(linestr, OGRGeometry(ls.wkt))
+            self.assertNotEqual(linestr, prev)
             self.assertRaises(OGRIndexError, linestr.__getitem__, len(linestr))
             prev = linestr
 
@@ -170,8 +183,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             self.assertEqual('MULTILINESTRING', mlinestr.geom_name)
             self.assertEqual(mls.n_p, mlinestr.point_count)
             self.assertEqual(mls.coords, mlinestr.tuple)
-            self.assertEqual(True, mlinestr == OGRGeometry(mls.wkt))
-            self.assertEqual(True, mlinestr != prev)
+            self.assertEqual(mlinestr, OGRGeometry(mls.wkt))
+            self.assertNotEqual(mlinestr, prev)
             prev = mlinestr
             for ls in mlinestr:
                 self.assertEqual(2, ls.geom_type)
@@ -186,16 +199,16 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             #self.assertEqual(101, lr.geom_type.num)
             self.assertEqual('LINEARRING', lr.geom_name)
             self.assertEqual(rr.n_p, len(lr))
-            self.assertEqual(True, lr == OGRGeometry(rr.wkt))
-            self.assertEqual(True, lr != prev)
+            self.assertEqual(lr, OGRGeometry(rr.wkt))
+            self.assertNotEqual(lr, prev)
             prev = lr
 
     def test07a_polygons(self):
         "Testing Polygon objects."
 
         # Testing `from_bbox` class method
-        bbox =  (-180,-90,180,90)
-        p = OGRGeometry.from_bbox( bbox )
+        bbox = (-180, -90, 180, 90)
+        p = OGRGeometry.from_bbox(bbox)
         self.assertEqual(bbox, p.extent)
 
         prev = OGRGeometry('POINT(0 0)')
@@ -213,8 +226,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             self.assertAlmostEqual(p.centroid[1], y, 9)
 
             # Testing equivalence
-            self.assertEqual(True, poly == OGRGeometry(p.wkt))
-            self.assertEqual(True, poly != prev)
+            self.assertEqual(poly, OGRGeometry(p.wkt))
+            self.assertNotEqual(poly, prev)
 
             if p.ext_ring_cs:
                 ring = poly[0]
@@ -230,26 +243,16 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
         # Both rings in this geometry are not closed.
         poly = OGRGeometry('POLYGON((0 0, 5 0, 5 5, 0 5), (1 1, 2 1, 2 2, 2 1))')
         self.assertEqual(8, poly.point_count)
-        print "\nBEGIN - expecting IllegalArgumentException; safe to ignore.\n"
-        try:
-            c = poly.centroid
-        except OGRException:
-            # Should raise an OGR exception, rings are not closed
-            pass
-        else:
-            self.fail('Should have raised an OGRException!')
-        print "\nEND - expecting IllegalArgumentException; safe to ignore.\n"
+        with self.assertRaises(OGRException):
+            poly.centroid
 
-        # Closing the rings -- doesn't work on GDAL versions 1.4.1 and below:
-        # http://trac.osgeo.org/gdal/ticket/1673
-        if GDAL_VERSION <= (1, 4, 1): return
         poly.close_rings()
-        self.assertEqual(10, poly.point_count) # Two closing points should've been added
+        self.assertEqual(10, poly.point_count)  # Two closing points should've been added
         self.assertEqual(OGRGeometry('POINT(2.5 2.5)'), poly.centroid)
 
     def test08_multipolygons(self):
         "Testing MultiPolygon objects."
-        prev = OGRGeometry('POINT(0 0)')
+        OGRGeometry('POINT(0 0)')
         for mp in self.geometries.multipolygons:
             mpoly = OGRGeometry(mp.wkt)
             self.assertEqual(6, mpoly.geom_type)
@@ -297,7 +300,7 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             self.assertEqual(4269, mpoly.srid)
             self.assertEqual('NAD83', mpoly.srs.name)
 
-            # Incrementing through the multipolyogn after the spatial reference
+            # Incrementing through the multipolygon after the spatial reference
             # has been re-assigned.
             for poly in mpoly:
                 self.assertEqual(mpoly.srs.wkt, poly.srs.wkt)
@@ -306,7 +309,7 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
                     # Changing each ring in the polygon
                     self.assertEqual(32140, ring.srs.srid)
                     self.assertEqual('NAD83 / Texas South Central', ring.srs.name)
-                    ring.srs = str(SpatialReference(4326)) # back to WGS84
+                    ring.srs = str(SpatialReference(4326))  # back to WGS84
                     self.assertEqual(4326, ring.srs.srid)
 
                     # Using the `srid` property.
@@ -358,8 +361,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             d1 = OGRGeometry(self.geometries.diff_geoms[i].wkt)
             d2 = a.difference(b)
             self.assertEqual(d1, d2)
-            self.assertEqual(d1, a - b) # __sub__ is difference operator
-            a -= b # testing __isub__
+            self.assertEqual(d1, a - b)  # __sub__ is difference operator
+            a -= b  # testing __isub__
             self.assertEqual(d1, a)
 
     def test11_intersection(self):
@@ -368,11 +371,11 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             a = OGRGeometry(self.geometries.topology_geoms[i].wkt_a)
             b = OGRGeometry(self.geometries.topology_geoms[i].wkt_b)
             i1 = OGRGeometry(self.geometries.intersect_geoms[i].wkt)
-            self.assertEqual(True, a.intersects(b))
+            self.assertTrue(a.intersects(b))
             i2 = a.intersection(b)
             self.assertEqual(i1, i2)
-            self.assertEqual(i1, a & b) # __and__ is intersection operator
-            a &= b # testing __iand__
+            self.assertEqual(i1, a & b)  # __and__ is intersection operator
+            a &= b  # testing __iand__
             self.assertEqual(i1, a)
 
     def test12_symdifference(self):
@@ -383,8 +386,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             d1 = OGRGeometry(self.geometries.sdiff_geoms[i].wkt)
             d2 = a.sym_difference(b)
             self.assertEqual(d1, d2)
-            self.assertEqual(d1, a ^ b) # __xor__ is symmetric difference operator
-            a ^= b # testing __ixor__
+            self.assertEqual(d1, a ^ b)  # __xor__ is symmetric difference operator
+            a ^= b  # testing __ixor__
             self.assertEqual(d1, a)
 
     def test13_union(self):
@@ -395,8 +398,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             u1 = OGRGeometry(self.geometries.union_geoms[i].wkt)
             u2 = a.union(b)
             self.assertEqual(u1, u2)
-            self.assertEqual(u1, a | b) # __or__ is union operator
-            a |= b # testing __ior__
+            self.assertEqual(u1, a | b)  # __or__ is union operator
+            a |= b  # testing __ior__
             self.assertEqual(u1, a)
 
     def test14_add(self):
@@ -415,10 +418,11 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
             mp3 = OGRGeometry('MultiPolygon')
 
             for poly in mpoly:
-                mp1.add(poly) # Adding a geometry at a time
-                mp2.add(poly.wkt) # Adding WKT
-            mp3.add(mpoly) # Adding a MultiPolygon's entire contents at once.
-            for tmp in (mp1, mp2, mp3): self.assertEqual(mpoly, tmp)
+                mp1.add(poly)  # Adding a geometry at a time
+                mp2.add(poly.wkt)  # Adding WKT
+            mp3.add(mpoly)  # Adding a MultiPolygon's entire contents at once.
+            for tmp in (mp1, mp2, mp3):
+                self.assertEqual(mpoly, tmp)
 
     def test15_extent(self):
         "Testing `extent` property."
@@ -446,9 +450,8 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
 
     def test17_pickle(self):
         "Testing pickle support."
-        import cPickle
         g1 = OGRGeometry('LINESTRING(1 1 1,2 2 2,3 3 3)', 'WGS84')
-        g2 = cPickle.loads(cPickle.dumps(g1))
+        g2 = pickle.loads(pickle.dumps(g1))
         self.assertEqual(g1, g2)
         self.assertEqual(4326, g2.srs.srid)
         self.assertEqual(g1.srs.wkt, g2.srs.wkt)
@@ -480,13 +483,5 @@ class OGRGeomTest(unittest.TestCase, TestDataMixin):
 
     def test19_equivalence_regression(self):
         "Testing equivalence methods with non-OGRGeometry instances."
-        self.assertNotEqual(None, OGRGeometry('POINT(0 0)'))
-        self.assertEqual(False, OGRGeometry('LINESTRING(0 0, 1 1)') == 3)
-
-def suite():
-    s = unittest.TestSuite()
-    s.addTest(unittest.makeSuite(OGRGeomTest))
-    return s
-
-def run(verbosity=2):
-    unittest.TextTestRunner(verbosity=verbosity).run(suite())
+        self.assertIsNotNone(OGRGeometry('POINT(0 0)'))
+        self.assertNotEqual(OGRGeometry('LINESTRING(0 0, 1 1)'), 3)
